function act = simDM_ACT(hdr_rate,is_coax,pcb_cutoff)
% This is simulation code provided to help with the development of
% IEEE 802.3dm.
%
% This code is provided for reference to allow independent evaluation
% of the accuracy and applicability of the simulation results shared in
% IEEE 802.3dm presentations by the autor.
%
% Written by Ragnar Jonsson, affiliated with Marvell Technology, Inc.
% Version 1.0, December 16th, 2024
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.

  if(nargin < 1)
    hdr_rate = 10;
  end
  if(nargin < 2)
    is_coax = 1;
  end
  if(nargin < 3)
    pcb_cutoff = 10;
  end

  %%% create the simulation data structure %%%
  act = link();

  %%% set over sampling ration for both high and low data rate (22.5GHz)%%%
  act.hdr_oversampling = 40/hdr_rate;
  act.ldr_oversampling = 192;

  %%% emulate LDR Tx and Rx analog filters as 4th order Butterworth filters %%%
  [b,a]=butter(4,4.5/act.ldr_oversampling);
  act.lkp.afe.tx_filter.a = a;
  act.lkp.afe.tx_filter.b = b;
  act.dut.afe.rx_filter.a = a;
  act.dut.afe.rx_filter.b = b;

  %%% emulate HDR Tx and Rx analog filters as 4th order Butterworth filters %%%
  [b,a]=butter(4,1/act.hdr_oversampling);
  act.lkp.afe.rx_filter.a = a;
  act.lkp.afe.rx_filter.b = b;
  act.dut.afe.tx_filter.a = a;
  act.dut.afe.tx_filter.b = b;

  %%% emulate PCB trace as simple RC circuit %%%
  b_pcb = [0.25 0.25];
  a_pcb = [1 -0.5];

  %%% emulate AC-cap as RC circuit %%%
  R = 100;
  C = 10e-9; % 10nF, per Table 149C–2
  f_s = 22.5e9;
  f_0 = 1/(C*R);
  b_cap = 2*f_s/(2*f_s + f_0)*[1 -1];
  a_cap = [1 -(2*f_s-f_0)/(2*f_s+f_0)];

  %%% emulate PoC HP characteristics as second order Butterworth filters %%%
  [b_poc,a_poc] = butter(2,pcb_cutoff*2/22.5e3,'high');

  %%% emulate the total PCB transfer function as product of component transfer %%%
  a = conv(a_pcb,conv(a_poc,a_cap));
  b = conv(b_pcb,conv(b_poc,b_cap));
  if(is_coax)
    tx_scale = 0.5;
  else
    tx_scale = 1;
  end
  act.lkp.pcb.tx_filter.a = a;
  act.lkp.pcb.tx_filter.b = b*tx_scale;
  act.lkp.pcb.rx_filter.a = a;
  act.lkp.pcb.rx_filter.b = b;
  act.dut.pcb = act.lkp.pcb;

  %%% map LDR bits to Manchester code %%%
  act.lkp.pma.mapper.hold = [1 -1];

  %%% emulate LDR ADC as Zero-order-Hold %%%
  act.lkp.afe.dac.hold = ones(1,act.ldr_oversampling/2)*0.5;

  %%% emulate HDR ADC as Zero-order-Hold %%%
  act.dut.afe.dac.hold = ones(1,act.hdr_oversampling)*0.5;

end
